import dayjs from "dayjs"

function mustMarshal(v, s) {
    let value = null
    switch (s) {
        case 'integer':
            if (v instanceof Date) {
                value = v.getTime() / 1000
            } else {
                value = parseInt(v)
                value = isNaN(value) ? 0 : value
            }
            break
        case 'decimal':
        case 'float':
            value = parseFloat(v)
            value = isNaN(value) ? 0 : value
            break
        case 'boolean':
            value = Boolean(v)
            break
        case 'string':
            if (v instanceof Date) {
                value = dayjs(v).format("YYYY-MM-DD HH:mm:ss")
            } else if (typeof v === 'object') {
                value = JSON.stringify(v)
            } else {
                value = v + ''
            }
            break
        default:
            value = v
    }
    return value
}

/**
 * 把对象转换成时间对象
 * @param {Object} v 
 * @returns 
 */
export function toDate(v) {
    if (typeof v === 'string') {
        return dayjs(v, ['YYYY-MM-DD HH:mm:ss', 'YYYY-MM-DD']).toDate();
    } else if (typeof v === 'number') {
        if (v / 1000000000 < 1000) {
            v = v * 1000;
        }
        return new Date(v);
    }
    return v;
}

/**
 *  模型数据编码为数据库数
 * @param {Object} raw 
 * @param {Array} schemas 
 * @returns 
 */
export function encode(raw, schemas, scenario) {
    let data = {}
    let excludes = []
    if (!raw || !schemas) {
        return data
    }
    scenario = scenario || 'unknown';
    for (let schema of schemas) {
        if (schema.attribute.live.enable && schema.attribute.live.type == 'cascader') {
            let columns = schema.attribute.live.columns
            //级联数据如果没有配置字段，那么对结果进行JSON编码处理
            if (!Array.isArray(columns) || columns.length == 0) {
                data[schema.column] = JSON.stringify(raw[schema.column])
            } else {
                let vals = raw[schema.column]
                if (vals && Array.isArray(vals)) {
                    for (let i in vals) {
                        if (columns[i]) {
                            //级联数据所有数据都必须是同一个类型的数据
                            data[columns[i]] = mustMarshal(vals[i], schema.type)
                            excludes.push(columns[i])
                        }
                    }
                }
            }
        } else if (['multiSelect'].indexOf(schema.format) > -1) {
            try {
                if (typeof raw[schema.column] === 'object') {
                    data[schema.column] = mustMarshal(
                        JSON.stringify(raw[schema.column]),
                        schema.type
                    )
                }
            } catch (e) { }
        } else if (['date', 'datetime', 'timestamp'].indexOf(schema.format) > -1) {
            let value = raw[schema.column];
            if (Array.isArray(value)) {
                let ps = []
                for (let i in value) {
                    let row = value[i];
                    ps.push(mustMarshal(toDate(row), schema.type))
                }
                data[schema.column] = ps.join('/')
            } else {
                data[schema.column] = mustMarshal(toDate(value), schema.type)
            }
        } else {
            //先前已经有过赋值的列数据
            if (excludes.indexOf(schema.column) > -1) {
                continue
            }
            //如果是bool类型的变量，并且值为空不进行类型转换
            if (['boolean', 'bool'].indexOf(schema.format) > -1) {
                if (typeof raw[schema.column] === 'undefined') {
                    continue
                }
                if (typeof raw[schema.column] === 'string' && raw[schema.column] == '') {
                    continue
                }
            }
            if (raw[schema.column]) {
                data[schema.column] = mustMarshal(raw[schema.column], schema.type)
            } else {
                data[schema.column] = mustMarshal(
                    scenario === 'create' ? schema.attribute.default_value : '',
                    schema.type
                )
            }
        }
    }
    return data
}

/**
 * 数据库数据解码为模型数据
 * @param {Object} raw 
 * @param {Array} schemas 
 * @returns 
 */
export function decode(raw, schemas, scenario) {
    let data = {}
    scenario = scenario || 'unknown';
    if (!raw || !schemas) {
        return data
    }
    for (let schema of schemas) {
        if (
            schema.attribute.live.enable &&
            schema.attribute.live.type == 'cascader'
        ) {
            let columns = schema.attribute.live.columns
            //级联数据如果没有配置字段，那么对结果进行JSON解码处理
            if (!Array.isArray(columns) || columns.length == 0) {
                if (raw[schema.column]) {
                    data[schema.column] = JSON.parse(raw[schema.column])
                } else {
                    data[schema.column] = []
                }
            } else {
                data[schema.column] = []
                for (let column of columns) {
                    let v = raw[column]
                    if (!v) {
                        break
                    }
                    data[schema.column].push(mustMarshal(v, schema.type))
                }
            }
        } else if (
            (schema.attribute.live.enable &&
                schema.attribute.live.type == 'dropdown') ||
            schema.format == 'dropdown'
        ) {
            //强制重新设置字段的format
            if (schema.format !== 'dropdown') {
                schema.format = 'dropdown';
            }
            //下拉列转成字符串显示
            let v = null;
            if (raw.hasOwnProperty(schema.column)) {
                v = raw[schema.column];
            } else if (scenario === 'create') {
                v = schema.attribute.default_value;
            }
            data[schema.column] = mustMarshal(v || '', 'string')
        } else if (schema.format === 'multiSelect') {
            try {
                if (typeof raw[schema.column] === 'string') {
                    data[schema.column] = JSON.parse(raw[schema.column])
                }
            } catch (e) { }
        } else if (['time', 'date', 'datetime', 'timestamp'].indexOf(schema.format) > -1) {
            if (raw.hasOwnProperty(schema.column)) {
                let v = raw[schema.column];
                if (v) {
                    if (typeof v === 'string' && v.indexOf('/') > -1) {
                        let ss = v.split('/', 2);
                        let st = toDate(ss[0]);
                        let et = toDate(ss[1]);
                        switch (schema.format) {
                            case 'time':
                                data[schema.column] = [dayjs(st).format('HH:mm'), dayjs(et).format('HH:mm')]
                                break;
                            case 'date':
                                data[schema.column] = [dayjs(st).format('YYYY-MM-DD'), dayjs(et).format('YYYY-MM-DD')];
                                break;
                            default:
                                data[schema.column] = [dayjs(st).format('YYYY-MM-DD HH:mm:ss'), dayjs(et).format('YYYY-MM-DD HH:mm:ss')];
                                break
                        }
                    } else {
                        let ts = toDate(v);
                        switch (schema.format) {
                            case 'time':
                                data[schema.column] = dayjs(ts).format('HH:mm');
                                break;
                            case 'date':
                                data[schema.column] = dayjs(ts).format('YYYY-MM-DD');
                                break;
                            default:
                                data[schema.column] = dayjs(ts).format('YYYY-MM-DD HH:mm:ss');
                                break
                        }
                    }

                }
            }
        } else {
            let v = null;
            if (raw.hasOwnProperty(schema.column)) {
                v = raw[schema.column];
            } else if (scenario === 'create') {
                v = schema.attribute.default_value;
            }
            data[schema.column] = mustMarshal(v || '', schema.type)
        }
    }
    return data
}